######### F. prausnitzii prophages####################################
######### A.N Shkoporov & C.B. Fitzgerald ©2018#######################
#Load requred libraries
library(gplots)
library(reshape)
library(ggplot2)
library(grid)
library(gridExtra)
library(RColorBrewer)
library(ggdendro)
library(ggrepel)
library(pvclust)

my_hclust_fun = function(x) {hclust(x, method = "ward.D2")}

#Read in data
fp_groups_in_prophages = read.table(file = "fp_groups_in_prophages.txt", header = FALSE)
colnames(fp_groups_in_prophages) = c("protein", "fp_group", "perc_identity")

regmatch1 = regexpr("^.*_phage_[0-9]{1,4}[a-z]*(corr)*_", as.character(fp_groups_in_prophages$protein)) #Match prophage id
regmatch2 = regexpr("_[0-9]{1,4}$", as.character(fp_groups_in_prophages$protein)) #Match protein id

fp_groups_in_prophages$prophage_id = regmatches(as.character(fp_groups_in_prophages$protein), regmatch1) #Add prophage id column
fp_groups_in_prophages$protein_id = regmatches(as.character(fp_groups_in_prophages$protein), regmatch2) #Add protein id column
fp_groups_in_prophages$prophage_id = gsub("_$", "", fp_groups_in_prophages$prophage_id) #Remove trailing underscores
fp_groups_in_prophages$protein_id = gsub("^_", "", fp_groups_in_prophages$protein_id) #Remove leading underscores

write.table(levels(fp_groups_in_prophages$fp_group), file = "fp_groups_of_interest.txt", col.names = FALSE, row.names = FALSE, quote = FALSE)

#Demovir output
demovir = read.table(file = "DemoVir_assignments.txt", header = TRUE, row.names = 1)
demovir$length_kb =
  as.numeric(gsub("kb", "", gsub("_.*_", "", sapply(strsplit(rownames(demovir), split = "phage"), "[[", 2))))
regmatch3 = regexpr("^.*_phage_[0-9]{1,4}[a-z]*(corr)*_", rownames(demovir))
rownames(demovir) = regmatches(rownames(demovir), regmatch3)
rownames(demovir) = gsub("_$", "", rownames(demovir))

#Create crosstab with orthologous groups (contingency table)
fp_groups_crosstab = as.data.frame.matrix(table(fp_groups_in_prophages[,c(2,4)]))
fp_groups_crosstab.bin = fp_groups_crosstab 
fp_groups_crosstab.bin[fp_groups_crosstab > 0] = 1

#Virsorter output
virsorter = read.table(file = "virsorter_positive.ids", header = FALSE)
regmatch4 = regexpr("^.*_phage_[0-9]{1,4}[a-z]*(corr)*_", virsorter$V1)
virsorter$V1 = regmatches(virsorter$V1, regmatch4)
virsorter$V1 = gsub("_$", "", virsorter$V1)

#GC content output
gccontent = read.table(file = "all_prophages_gc_content.txt", header = FALSE)
regmatch5 = regexpr("^.*_phage_[0-9]{1,4}[a-z]*(corr)*_", gccontent$V1)
rownames(gccontent) = regmatches(gccontent$V1, regmatch5)
rownames(gccontent) = gsub("_$", "", rownames(gccontent))

rm(regmatch1, regmatch2, regmatch3, regmatch4, regmatch5)

#Read in orthologous data for orthologous protein groups
#COG data
fp_groups_import = read.table(file = "fpr_pangenome_COG_annotation.txt", sep = "\t", row.names = 1, header = TRUE)
rownames(fp_groups_import) = gsub("_", "", rownames(fp_groups_import))
fp_groups_data = fp_groups_import[rownames(fp_groups_crosstab),1:3]

#pVOGs data
pvogs_import = read.table(file = "pvogs_final.txt", sep = "\t", header = FALSE)

#Add data to proteins table
fp_groups_in_prophages$COG_code = fp_groups_data[fp_groups_in_prophages$fp_group, "COG_number"]
fp_groups_in_prophages$COG_function = fp_groups_data[fp_groups_in_prophages$fp_group, "name"]
fp_groups_in_prophages$COG_category = fp_groups_data[fp_groups_in_prophages$fp_group, "subcategory"]
fp_groups_in_prophages[fp_groups_in_prophages == 0] = NA

fp_groups_in_prophages$pVOGs = NA
for (i in rownames(fp_groups_in_prophages)) {
  fp_groups_in_prophages[i, "pVOGs"] = 
    paste(as.character(pvogs_import[which(pvogs_import$V1 == as.character(fp_groups_in_prophages[i, "fp_group"])), "V2"]), collapse = " ")
}

rm(fp_groups_import, fp_groups_data, pvogs_import)

#Create a 'per prophage' data table
prophage_data = data.frame(
  row.names = unique(fp_groups_in_prophages$prophage_id),
  length_kb = demovir[unique(fp_groups_in_prophages$prophage_id), "length_kb"],
  size_category = cut(demovir[unique(fp_groups_in_prophages$prophage_id), "length_kb"],
      breaks = c(0, 30, 60, 120), labels = c("small", "medium", "large"), include.lowest = TRUE),
  demovir = demovir[unique(fp_groups_in_prophages$prophage_id), "Family"],
  virsorter = FALSE,
  gc_content = gccontent[unique(fp_groups_in_prophages$prophage_id), "V2"],
  strain = NA,
  cluster = NA,
  n_genes = 0,
  n_COGs = 0,
  n_pVOGs = 0,
  COGs_list = NA,
  pVOGs_list = NA,
  COG_functions = NA,
  pVOG_functions = NA
)
prophage_data[virsorter$V1,"virsorter"] = TRUE

for (i in rownames(prophage_data)) {
  prot_id_temp = which(fp_groups_in_prophages$prophage_id == i)
  prophage_data[i, "strain"] = sapply(strsplit(fp_groups_in_prophages[prot_id_temp[1], "prophage_id"], split = "_phage_"), "[[", 1)
  prophage_data[i, "n_genes"] = length(prot_id_temp)
  prophage_data[i, "n_COGs"] = length(as.character(na.exclude(fp_groups_in_prophages[prot_id_temp, "COG_code"])))
  prophage_data[i, "n_pVOGs"] = length(as.character(fp_groups_in_prophages[prot_id_temp, "pVOGs"])[as.character(fp_groups_in_prophages[prot_id_temp, "pVOGs"]) != ""])
  prophage_data[i, "COGs_list"] = paste(as.character(na.exclude(fp_groups_in_prophages[prot_id_temp, "COG_code"])), collapse = " ")
  prophage_data[i, "pVOGs_list"] = paste(as.character(fp_groups_in_prophages[prot_id_temp, "pVOGs"])[as.character(fp_groups_in_prophages[prot_id_temp, "pVOGs"]) != ""],
                                         collapse = " ")
  prophage_data[i, "COG_functions"] = paste(as.character(na.exclude(fp_groups_in_prophages[prot_id_temp, "COG_function"])), collapse = "; ")
}
rm(demovir, virsorter, gccontent)

#Distance matrix with percentages of shared genes
perc_identity = function(m) 
{
  perc_shared_mtx = matrix(, nrow = ncol(m), ncol = ncol(m))
  colnames(perc_shared_mtx) = colnames(m)
  rownames(perc_shared_mtx) = colnames(m)
  
  for (i in colnames(m))
  {
    orthologs_present = which(m[,i] == 1)
    for (k in colnames(m))
    {
      perc_shared_mtx[i,k] = sum(m[orthologs_present,k])/length(orthologs_present)
    }
    
  }
  return(perc_shared_mtx)
}

#Distance matrix and clustering
fp_groups_crosstab.bin = fp_groups_crosstab.bin[,rownames(prophage_data)[prophage_data$virsorter]] #Keep only Virsorter-positive
fp_groups_crosstab.bin = fp_groups_crosstab.bin[apply(fp_groups_crosstab.bin, 1, function(x) any(x > 0)),]

perc_orthologs_shared = perc_identity(fp_groups_crosstab.bin)
perc_orthologs_shared_dist = as.dist(1-perc_orthologs_shared)
perc_orthologs_shared_clust = hclust(perc_orthologs_shared_dist, method = "ward.D2")
perc_orthologs_shared = perc_orthologs_shared[perc_orthologs_shared_clust$order,perc_orthologs_shared_clust$order]

#perc_orthologs_shared_pvclust = pvclust(perc_orthologs_shared, method.hclust="ward.D2", method.dist = "euclidean")
#perc_orthologs_shared_pvpick = pvpick(perc_orthologs_shared_pvclust, alpha = 0.95)

#Append clustering to prophage_data
#for (i in 1:length(perc_orthologs_shared_pvpick$clusters)) {
#  prophage_data[perc_orthologs_shared_pvpick$clusters[[i]], "cluster"] = i
#}
#prophage_data[is.na(prophage_data$cluster), "cluster"] = "orphan"
prophage_data$cluster = cutree(perc_orthologs_shared_clust, k = 12)[rownames(prophage_data)]
prophage_data$cluster = as.factor(prophage_data$cluster)

prophage_data_all = prophage_data #Remove incomplete regions from prophage_data table.
prophage_data = prophage_data[prophage_data$virsorter,]

#prophage_data[prophage_data$cluster == "2", "cluster"] = "orphan"
write.csv(prophage_data, file = "prophage_data.csv")

#Create crosstab with clusters of prophages (contingency table)
prophage_clusters_crosstab = as.data.frame.matrix(table(prophage_data[,c("strain", "cluster")]))

#Phage families to annotate heatmap
phage_families_order = c()
phage_clusters_size = c()
for (i in colnames(prophage_clusters_crosstab)) {
  if (i == "orphan") {
    phage_families_order = c(phage_families_order, NA)
    phage_clusters_size = c(phage_clusters_size, NA)
  }
  else {
    phage_families_order = c(phage_families_order,
                             names(table(as.character(prophage_data[prophage_data$cluster == i, "demovir"]))[1])) 
    phage_clusters_size = c(phage_clusters_size, max(prophage_data[prophage_data$cluster == i, "length_kb"]))
  }
}
phage_families_colors = c("purple", "green", "yellow")[as.numeric(as.factor(phage_families_order))]
colnames(prophage_clusters_crosstab) = paste("Cluster", colnames(prophage_clusters_crosstab))

#Visualization of clusters based on shared genes
perc_orthologs_shared_melt = melt(perc_orthologs_shared)
perc_orthologs_shared_melt$X1 = factor(perc_orthologs_shared_melt$X1, levels = unique(perc_orthologs_shared_melt$X1))
perc_orthologs_shared_melt$X2 = factor(perc_orthologs_shared_melt$X2, levels = unique(perc_orthologs_shared_melt$X2))
perc_orthologs_shared_melt$value = cut(perc_orthologs_shared_melt$value, breaks = c(0, 0.2, 0.4, 0.6, 0.8, 1),
    labels = c("0-20", "20-40", "40-60", "60-80", "80-100"), include.lowest = TRUE)

perc_orthologs_shared_heatmap = ggplot(data = perc_orthologs_shared_melt,
    aes(x = X1, y = X2, fill = value, label = round(value*100, digits = 1))) +
  geom_tile() + theme(text = element_text(size = 8), axis.text.x = element_text(angle = 45, hjust = 1)) +
  xlab("") + ylab("") + scale_fill_manual(values = c("white", "#F8766D", "#7CAE00", "#00BFC4", "#C77CFF"))
#annotate("text", x = 1:244, y = 20, label = as.character(contig_metadata.df$clusters_kopt[perc_orthologs_shared_clust$order]))

perc_orthologs_shared_dendro = dendro_data(perc_orthologs_shared_clust, type="rectangle")
perc_orthologs_shared_dendroplot = ggplot(segment(perc_orthologs_shared_dendro)) + theme_void() +
  geom_segment(aes(x=x, y=y, xend=xend, yend=yend))

#Visualization of prophages
prophage_stats = ggplot(prophage_data_all, aes(x = length_kb, y = gc_content, colour = n_pVOGs, size = virsorter, shape = demovir, label = cluster, group = cluster)) +
  theme_classic() + geom_point() + geom_text_repel(show.legend = FALSE) +
  ggtitle("Statistics of prophage size, family assignment, G+C and gene content") +
  theme(plot.margin = margin(0, 0, 15, 0, "cm"))

#PDF
pdf(file="prophage_orthomcl_fig1.pdf", paper="A4r", width=11, height=8.5)
grid.newpage()
print(perc_orthologs_shared_dendroplot,
      vp = viewport(width = 0.89, height = 0.2, y = 0.89, x = 0.510))
print(perc_orthologs_shared_heatmap,
      vp = viewport(width = 1, height = 0.8, y = 0.4))
dev.off()

pdf(file="prophage_orthomcl_fig2.pdf", paper="A4r", width=11, height=8.5)
par(lwd = 2)
#plot(perc_orthologs_shared_pvclust)
#pvrect(perc_orthologs_shared_pvclust, alpha = 0.95)
plot(perc_orthologs_shared_clust, hang = -1, cex = 0.7)
rect.hclust(perc_orthologs_shared_clust, k = 12, border = "red")
dev.off()

pdf(file="prophage_orthomcl_fig3a.pdf", paper="A4", width=8.5, height=11)
prophage_stats
dev.off()

pdf(file="prophage_orthomcl_fig3b.pdf", paper="A4", width=8.5, height=11)
par(lend = 1, lwd = 2)
heatmap.2(as.matrix(prophage_clusters_crosstab),
          col = c("white", "blue", "red"),
          trace = "none", hclustfun = my_hclust_fun,
          ColSideColors = phage_families_colors,
          density.info = "none",
          key = FALSE,
          keysize = 1,
          margins = c(40, 10),
          cexCol = 1.5,
          cexRow = 1.5)
legend(0,0.9, legend = levels(as.factor(phage_families_order)), col = c("purple", "green", "yellow"),
       lty = 1, lwd = 5, cex = 0.8, bg = "white")
dev.off()