#Calculation of synteny score in Faecalibacterium genomes. Assumes circular genome map. A.Shkoporov 2018
library(ggplot2)

#Read in alignment blocks data and remove strain specific sequence part
backbone_df_long = read.table(list.files(pattern = ".backbone"), header = TRUE) #Change this
synteny_output = matrix(nrow = ncol(combn(c(1:(ncol(backbone_df_long)/2)), 2)), ncol = 2)

#Go through all combinations of strains

for(k in 1:ncol(combn(c(1:(ncol(backbone_df_long)/2)), 2))) {
  
  strains = combn(c(1:(ncol(backbone_df_long)/2)), 2)[,k]
  backbone_df = backbone_df_long[,c(strains[1]*2-1,strains[1]*2,strains[2]*2-1,strains[2]*2)]
  
  backbone_df = backbone_df[!apply(backbone_df_long == 0, 1, function(x) any(x)),]
  colnames(backbone_df) = c("seq0_leftend", "seq0_rightend", "seq1_leftend", "seq1_rightend")
  
  #Total length of sequence in alignment file
  genome0_aligned = max(backbone_df$seq0_rightend)
  genome1_aligned = max(backbone_df$seq1_rightend)
  
  #Calculate block lengths and assign numeric id to each block
  backbone_df$block_length0 = abs(backbone_df$seq0_rightend - backbone_df$seq0_leftend)
  backbone_df$block_length1 = abs(backbone_df$seq1_rightend - backbone_df$seq1_leftend)
  backbone_df = backbone_df[backbone_df$block_length0 > 1000,] #Filter blocks by size
  #backbone_df$block_id = as.numeric(rownames(backbone_df)) #!!!
  backbone_df$block_id = 1:nrow(backbone_df)
  rownames(backbone_df) = backbone_df$block_id
  backbone_df$synteny = FALSE
  
  #Reorder blocks for each of the genomes to restore the original order
  block_sequence0 = backbone_df[order(abs(backbone_df$seq0_leftend)), "block_id"]
  block_sequence1 = backbone_df[order(abs(backbone_df$seq1_leftend)), "block_id"]
  block_sequences = data.frame(block_sequence1 = block_sequence1, block_sequence0 = block_sequence0)
  
  block_sequences = block_sequences[order(block_sequences$block_sequence1),]
  rownames(block_sequences) = block_sequences$block_sequence1
  rm(block_sequence0, block_sequence1)
  
  #Calculate block offset for Genome 0
  block_sequences[1,"offset"] = 1
  for (i in 2:nrow(block_sequences))
  {
    block_sequences[i,"offset"] = block_sequences[i,"block_sequence0"] - block_sequences[i-1,"block_sequence0"]
  }
  
  #Identify asyntenic blocks of sequence (blocks with offset != 1 as well as subsequent block (?..))
  asyntenic_blocks = block_sequences[abs(block_sequences$offset) != 1 & block_sequences$block_sequence0 != 1, "block_sequence1"]
  #asyntenic_blocks = unique(c(asyntenic_blocks, which(abs(block_sequences$offset) != 1 & block_sequences$block_sequence0 != 1)+1))
  block_sequences$asyntenic = FALSE
  block_sequences[asyntenic_blocks, "asyntenic"] = TRUE #!!!
  rm(asyntenic_blocks)
  
  #Remove solitary blocks
  #for (i in 2:nrow(block_sequences))
  #{
  #  if (!block_sequences[i,"asyntenic"] & block_sequences[i-1,"asyntenic"] & block_sequences[i+1,"asyntenic"])
  #  {block_sequences[i,"asyntenic"] = TRUE}
  #}
  
  #Append synteny data to backbone_df
  syntenic_blocks = block_sequences[!block_sequences$asyntenic, "block_sequence1"]
  backbone_df[match(syntenic_blocks, backbone_df$block_id), "synteny"] = TRUE
  backbone_df[1, "synteny"] = TRUE
  backbone_df[sign(backbone_df$seq0_leftend) != sign(backbone_df$seq1_leftend), "synteny"] = FALSE
  
  #Plot relative block sequences in the two genomes
  #plot(block_sequence0~block_sequence1, block_sequences, col = c("black", "red")[as.numeric(block_sequences$asyntenic)+1])
  #lines(block_sequence0~block_sequence1, block_sequences)

  #Plot relative block sequences in the two genomes after filtration
  #plot(block_sequence0~block_sequence1, block_sequences[syntenic_blocks,])
  #lines(block_sequence0~block_sequence1, block_sequences[syntenic_blocks,])
  
  #Save reciprocal synteny result
  synteny_output[k, 1] = sum(backbone_df[backbone_df$synteny, "block_length0"])/genome0_aligned
  synteny_output[k, 2] = sum(backbone_df[backbone_df$synteny, "block_length1"])/genome1_aligned
  
  #Make long version of block_sequences
  block_sequences$strains = paste("Strain", strains[1], "vs.", "strain", strains[2])
  if(k == 1) {block_sequences_all = block_sequences}
  else {block_sequences_all = rbind(block_sequences_all, block_sequences)}
  
}

print(synteny_output)

#ggplot
pdf(file="synteny.pdf", paper="A4r", width=11, height=8.5)
ggplot(block_sequences_all, aes(x = block_sequence0, y = block_sequence1, colour = asyntenic)) +
  theme_classic() + geom_point() + scale_colour_manual(values = c("black", "red")) +
  facet_wrap(~strains)
dev.off()

#Output syntenic blocks data into a file
write.table(synteny_output, file = "synteny_output.tab", sep = "\t", row.names = FALSE)
